﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Analysis;
using HeuristicLab.Common;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;
using HeuristicLab.Problems.DataAnalysis.Views;

namespace HeuristicLab.Algorithms.NCA.Views {
  [View("Dimensionality Reduction")]
  [Content(typeof(NCAClassificationSolution), IsDefaultView = false)]
  public partial class NCADimensionalityReductionView : DataAnalysisSolutionEvaluationView {
    private ViewHost viewHost = new ViewHost();
    private ScatterPlot scatterPlot = new ScatterPlot();

    public new NCAClassificationSolution Content {
      get { return (NCAClassificationSolution)base.Content; }
      set { base.Content = value; }
    }

    public NCADimensionalityReductionView() {
      InitializeComponent();
      viewHost.Visible = false;
      viewHost.Dock = DockStyle.Fill;
      splitContainer.Panel2.Controls.Add(viewHost);
      rangeComboBox.SelectedIndex = 0;
    }

    #region Register Content Events
    protected override void DeregisterContentEvents() {
      // TODO: Deregister your event handlers on the Content here
      base.DeregisterContentEvents();
    }
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      // TODO: Register your event handlers on the Content here
    }
    #endregion

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        scatterPlot.Rows.Clear();
        messageLabel.Visible = false;
        viewHost.Visible = true;
      } else {
        var matrix = Content.Model.TransformationMatrix;
        if (matrix.GetLength(1) != 2) {
          messageLabel.Visible = true;
          viewHost.Visible = false;
        } else {
          messageLabel.Visible = false;
          viewHost.Content = null;
          viewHost.Visible = true;
          UpdateScatterPlot();
          viewHost.Content = scatterPlot;
        }
      }
    }

    private void UpdateScatterPlot() {
      var matrix = Content.Model.TransformationMatrix;
      scatterPlot.Rows.Clear();
      var rows = Content.ProblemData.ClassValues.ToDictionary(c => c, c => new ScatterPlotDataRow(Content.ProblemData.GetClassName(c), string.Empty, Enumerable.Empty<Point2D<double>>()));

      IEnumerable<int> range = null;
      if (rangeComboBox.SelectedIndex == 0) range = Content.ProblemData.TrainingIndices;
      else if (rangeComboBox.SelectedIndex == 1) range = Content.ProblemData.TestIndices;
      else range = Enumerable.Range(0, Content.ProblemData.Dataset.Rows);

      foreach (var r in range) {
        var label = Content.ProblemData.Dataset.GetDoubleValue(Content.ProblemData.TargetVariable, r);
        var point = Scale(matrix, r).ToArray();
        rows[label].Points.Add(new Point2D<double>(point[0], point[point.Length > 1 ? 1 : 0]));
      }
      scatterPlot.Rows.AddRange(rows.Values);
    }

    private IEnumerable<double> Scale(double[,] matrix, int row) {
      var instance = Content.ProblemData.AllowedInputVariables.Select(v => Content.ProblemData.Dataset.GetDoubleValue(v, row));
      for (int j = 0; j < matrix.GetLength(1); j++) {
        int i = 0;
        yield return instance.Sum(x => x * matrix[i++, j]);
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      rangeComboBox.Enabled = Content != null && viewHost.Visible;
    }


    #region Event Handlers
    private void rangeComboBox_SelectedIndexChanged(object sender, System.EventArgs e) {
      if (Content != null) UpdateScatterPlot();
    }
    #endregion
  }
}
